# -*- coding: utf-8 -*-

from Queue import Queue
from netkit.contrib.tcp_client import TcpClient
import time
from thread import start_new_thread
from ..gw_box import GWBox
from ..log import logger
from .. import constants
from ..gw_pb2 import RspToUids, CloseUids


CONNECT_INTERVAL = 1


class Sender(object):

    msg_queue = None
    host = None
    port = None
    connect_interval = None

    client = None

    def __init__(self, host, port, connect_interval=None):
        self.msg_queue = Queue()
        self.host = host
        self.port = port
        self.connect_interval = connect_interval or CONNECT_INTERVAL

        self.client = TcpClient(GWBox, host, port)

    def start(self):
        start_new_thread(self._send_worker, ())
        start_new_thread(self._recv_worker, ())

    def write(self, msg):
        return self.msg_queue.put_nowait(msg)

    def write_to_uids(self, data_list):
        """
        格式为
        [(uids, box), ...]
        :param data_list:
        :return:
        """

        msg = RspToUids()

        for uids, box in data_list:
            row = msg.rows.add()
            row.buf = str(box.pack())
            row.uids.extend(uids)

        gw_box = GWBox()
        gw_box.cmd = constants.CMD_WORKER_RSP_TO_UIDS
        gw_box.body = msg.SerializeToString()

        self.write(gw_box.pack())

    def close_uids(self, uids):
        msg = CloseUids()
        msg.uids.extend(uids)

        gw_box = GWBox()
        gw_box.cmd = constants.CMD_WORKER_CLOSE_UIDS
        gw_box.body = msg.SerializeToString()

        self.write(gw_box.pack())

    def _send_worker(self):
        """
        处理发送消息
        :return:
        """
        while 1:
            # 永久等在这里就可以
            msg = self.msg_queue.get()

            while self.client.closed():
                # 要一直等到可以发送为止，否则消息会被丢失
                logger.debug('connection closed, sleep %d', self.connect_interval)
                time.sleep(self.connect_interval)

            self.client.write(msg)

    def _recv_worker(self):
        """
        recv负责判断是否closed，以及重连
        :return:
        """
        while 1:
            if self.client.closed():
                try:
                    logger.debug('connect to server')
                    self.client.connect()
                except KeyboardInterrupt, e:
                    # 中断
                    raise e
                except:
                    # 说明链接失败了
                    logger.debug('connect fail, sleep %d', self.connect_interval)
                    time.sleep(self.connect_interval)
                    continue

            # 说明这里是对的
            ret = self.client.read()
            if not ret:
                # 链接断开
                self.client.close()